'----------------------------------------------------------------------------------------------------------------------
'
' Demo and test harness for heater_fft
'
' Michael Rychlik 2011-01-22
'
' This file is released under the terms of the MIT license. See below.
'
' 2010-12-16    v0.1  Initial draft.
'
' 2010-12-18    v0.2  First working version.
'
' 2010-12-18    v0.3  For heater_fft v0.3
'
' 2010-12-21    v0.4  For heater_fft v0.4
'
' 2010-12-28    v0.5  For heater_fft v0.5
'
' 2011-01-14    v1.0  For heater_fft v1.0
'                     Added copyright and MIT licence statemets.
'
' 2011-01-14    v1.1  For heater_fft v1.1
'
' 2011-01-22    v2.0  Moved x and y buffers out of heater_fft.
'                     Added a mailbox interface.
'                     Added bit-reversal step as an option.
'                     Added magnitude calculation as an option.
'
'----------------------------------------------------------------------------------------------------------------------
OBJ
    'Set your platform configuration in userdef.spin
    'Ideally, your userdefs.spin can live in the library path,
    'and you can simply remove a new package's userdefs.spin
    'to enable your own definitions.
    def  : "userdefs.spin"

    fft  : "heater_fft.spin"

    'User interface port
    ser  : "FullDuplexSerialPlus"

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
CON
    _clkmode      = def#_clkmode
    _xinfreq      = def#_xinfreq

'----------------------------------------------------------------------------------------------------------------------
VAR
    'cos and sin parts of the signal to be analysed
    'Result is written back to here.
    'Just write input sammles to bx and zero all by.
    long bx[fft#FFT_SIZE]
    long by[fft#FFT_SIZE]

    'Mailbox interface to fft
    long fft_mailbox_cmd        'Command
    long fft_mailbox_bxp        'Address of x buffer
    long fft_mailbox_byp        'Address of y buffer

'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
PUB testFFT | startTime, endTime, i
'Test harness for the FFT
'
    'Start the terminal
    ser.start(def#conRxPin, def#conTxPin, def#conMode, def#conBaud)
    ser.str(string("heater_fft v2.0"))
    newline

    fft.start(@fft_mailbox_cmd)

    'Input some data
    fillInput

    'The Fourier transform, including bit-reversal reordering and magnitude converstion
    startTime := cnt
    fft.butterflies(fft#CMD_DECIMATE | fft#CMD_BUTTERFLY | fft#CMD_MAGNITUDE, @bx, @by)
    endTime := cnt

    'Print resulting spectrum
    printSpectrum

    ser.str(string("1024 point FFT plus magnitude calculation run time = "))
    ser.dec((endTime - startTime) / (clkfreq / 1000))
    ser.str(string("ms"))
    newline
    newline
    repeat

PUB print_x | i
    repeat i from 0 to fft#FFT_SIZE / 2
        ser.dec(bx[i])
        newline

PUB printSpectrum  | i, real, imag, magnitude
'Spectrum is available in first half of the buffers after FFT.
    ser.str(string("Freq. Magnitude"))
    newline
    repeat i from 0 to (fft#FFT_SIZE / 2)
        ser.dec(i)
        ser.str(string(" "))
'        ser.hex(real, 8)
'        ser.str(string(" "))
'        ser.hex(imag, 8)
'        ser.str(string(" "))
        ser.dec(bx[i])
        newline


PUB fillInput | k, r, sx, sy
'Fill buffer bx with samples of of an imput signal and clear by.
'WARNING: Keep the peek amplitude < 2048.
    repeat k from 0 to (fft#FFT_SIZE - 1)

        'Two frequencies of the waveform defined in input
        bx[k] := (~~input[(3*k) // 16] / 4) - 1
        bx[k] += (~~input[(5*k) // 16] / 4) - 1

        'The highest frequency
        if (k & 1)
          bx[k] += 200
        else'
          bx[k] += -200

        'A DC level
        bx[k] += 200

{
        '12 bit Random input
        sx := (?r & $FFF)
        if sx & $800
            sx |= $FFFFF000
        bx[k] := sx
}
{
        if k & %1000
            bx[k] := 1023
        else
            bx[k] := -1023
}
        'Clear Y array.
        by[k] := 0

PUB newline
    ser.tx(13)
    ser.tx(10)

'----------------------------------------------------------------------------------------------------------------------
DAT
'For testing define 16 samples  of an input wave form here.
input long 4096, 3784, 2896, 1567, 0, -1567, -2896, -3784, -4096, -3784, -2896, -1567, 0, 1567, 2896, 3784
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
'    This file is distributed under the terms of the The MIT License as follows:
'
'    Copyright (c) 2011 Michael Rychlik
'
'    Permission is hereby granted, free of charge, to any person obtaining a copy
'    of this software and associated documentation files (the "Software"), to deal
'    in the Software without restriction, including without limitation the rights
'    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
'    copies of the Software, and to permit persons to whom the Software is
'    furnished to do so, subject to the following conditions:
'
'    The above copyright notice and this permission notice shall be included in
'    all copies or substantial portions of the Software.
'
'    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
'    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
'    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
'    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
'    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
'    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
'    THE SOFTWARE.
'----------------------------------------------------------------------------------------------------------------------

'----------------------------------------------------------------------------------------------------------------------
'The end.

